from typing import Optional, Union

from pydantic import BaseModel

from config import SCHEMAS_SUCCESS_CODE, SCHEMAS_SUCCESS_STATUS, SCHEMAS_SUCCESS_MESSAGE, SCHEMAS_ERROR_CODE, SCHEMAS_ERROR_STATUS, SCHEMAS_ERROR_MESSAGE


class Schemas(BaseModel):
    """状态返回"""
    code: Optional[int] = SCHEMAS_SUCCESS_CODE
    status: Optional[str] = SCHEMAS_SUCCESS_STATUS
    message: Optional[str] = SCHEMAS_SUCCESS_MESSAGE
    data: Optional[Union[BaseModel, dict, list, str, bool, None]] = None


class SchemasError(BaseModel):
    """状态返回"""
    code: Optional[int] = SCHEMAS_ERROR_CODE
    status: Optional[str] = SCHEMAS_ERROR_STATUS
    message: Optional[str] = SCHEMAS_ERROR_MESSAGE
    data: Optional[Union[BaseModel, dict, list, str, bool, None]] = None


class SchemasPaginate(BaseModel):
    """分页"""
    items: Optional[list] = None  # 当前页的数据列表
    pages: Optional[int] = None  # 总页数
    total: Optional[int] = None  # 总条数
    limit: Optional[int] = None  # 页条数


class ModelScreenParams(BaseModel):
    """获取列表默认参数"""
    page: Optional[int] = 1
    limit: Optional[int] = 25
    where: Optional[Union[dict, list]] = []
    join: Optional[Union[dict, list]] = []
    order: Optional[list] = []


class ModelScreenParamsForAll(BaseModel):
    """获取列表默认参数"""
    where: Optional[Union[dict, list]] = []
    join: Optional[Union[dict, list]] = []
    order: Optional[list] = []



from sqlalchemy.orm import Session

from lsshu.internal.crud import CRUDTree
from lsshu.oauth.model import ModelOAuthRoles, ModelOAuthPermissions
from lsshu.oauth.role.schema import SchemasOAuthRoleStoreUpdate


class CRUDOAuthRole(CRUDTree):
    """用户表操作"""
    params_model = ModelOAuthRoles
    params_relationship = {
        "permissions": ModelOAuthPermissions
    }

    @classmethod
    def store(cls, db: Session, item: SchemasOAuthRoleStoreUpdate, **kwargs):
        if item.permissions:
            from lsshu.internal.crud import BaseCRUD
            _relation = BaseCRUD.all(db=db, model=ModelOAuthPermissions, where=("id", 'in_', item.permissions))
            item.scopes = " ".join([relation.scope for relation in _relation])
        return super(CRUDOAuthRole, cls).store(db=db, item=item, **kwargs)

    @classmethod
    def update(cls, db: Session, pk: int, item: SchemasOAuthRoleStoreUpdate, **kwargs):
        if item.permissions:
            from lsshu.internal.crud import BaseCRUD
            _relation = BaseCRUD.all(db=db, model=ModelOAuthPermissions, where=("id", 'in_', item.permissions))
            item.scopes = " ".join([relation.scope for relation in _relation])
        return super(CRUDOAuthRole, cls).update(db=db, pk=pk, item=item, **kwargs)



from fastapi import APIRouter

from lsshu.oauth.user.main import router as router_user
from lsshu.oauth.role.main import router as router_role
from lsshu.oauth.permission.main import router as router_permission
from lsshu.oauth.annex.main import router as router_annex

from config import OAUTH_OAUTH_ROUTER

router = APIRouter(**OAUTH_OAUTH_ROUTER)

router.include_router(router_user)
router.include_router(router_role)
router.include_router(router_permission)
router.include_router(router_annex)



from datetime import datetime
from typing import Optional, List
from pydantic import BaseModel

from lsshu.internal.schema import SchemasPaginate
from lsshu.oauth.permission.schema import SchemasOAuthPermissionThinResponse


class SchemasOAuthRoleResponse(BaseModel):
    """角色 返回"""
    id: int
    name: Optional[str] = None
    permissions: Optional[List[SchemasOAuthPermissionThinResponse]] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None

    class Config:
        orm_mode = True


class SchemasOAuthRolePaginateItem(SchemasPaginate):
    items: List[SchemasOAuthRoleResponse]


class SchemasOAuthRoleStoreUpdate(BaseModel):
    """授权角色 提交"""
    name: Optional[str] = None
    scopes: Optional[str] = None
    permissions: Optional[List[int]] = None


class SchemasParams(BaseModel):
    permissions: Optional[list] = None



from typing import List

from fastapi import APIRouter, Depends, Security
from sqlalchemy.orm import Session

from lsshu.internal.db import dbs
from lsshu.internal.depends import model_screen_params, model_post_screen_params, auth_user
from lsshu.internal.schema import ModelScreenParams, Schemas, SchemasError
from lsshu.oauth.user.schema import SchemasOAuthScopes

from lsshu.demo.crud import CRUD
from lsshu.demo.model import name as name
from lsshu.demo.schema import SchemasResponse, SchemasParams, SchemasPaginateItem, SchemasStoreUpdate

router = APIRouter(tags=["Demo"])
scopes = [name, ]


@router.get('/{}'.format(name), name="get {}".format(name))
async def get_models(db: Session = Depends(dbs), params: ModelScreenParams = Depends(model_screen_params),
                     auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.list" % name])):
    """
    :param db:
    :param params:
    :param auth:
    :return:
    """
    db_model_list = CRUD.paginate(db=db, screen_params=params)
    return Schemas(data=SchemasPaginateItem(**db_model_list))


@router.post('/{}.post'.format(name), name="get {}".format(name))
async def get_models(db: Session = Depends(dbs), params: ModelScreenParams = Depends(model_post_screen_params),
                     auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.list" % name])):
    """
    :param db:
    :param params:
    :param auth:
    :return:
    """
    db_model_list = CRUD.paginate(db=db, screen_params=params)
    return Schemas(data=SchemasPaginateItem(**db_model_list))


@router.get('/{}.params'.format(name), name="get {}".format(name))
async def params_models(db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.list" % name])):
    """
    :param db:
    :param auth:
    :return:
    """

    data = {}
    return Schemas(data=SchemasParams(**data))


@router.get('/{}/{{pk}}'.format(name), name="get {}".format(name))
async def get_model(pk: int, db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.get" % name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUD.first(db=db, pk=pk)
    if db_model is None:
        return SchemasError(message="Data Not Found")
    return Schemas(data=SchemasResponse(**db_model.to_dict()))


@router.post('/{}'.format(name), name="get {}".format(name))
async def store_model(item: SchemasStoreUpdate, db: Session = Depends(dbs),
                      auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.store" % name])):
    """
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUD.first(db=db, where=("name", item.name))
    if db_model is not None:
        return SchemasError(message="Data Already Registered")
    bool_model = CRUD.store(db=db, item=item)
    return Schemas(data=SchemasResponse(**bool_model.to_dict()))


@router.put("/{}/{{pk}}".format(name), name="update {}".format(name))
async def update_put_model(pk: int, item: SchemasStoreUpdate, db: Session = Depends(dbs),
                           auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.update" % name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUD.first(db=db, pk=pk)
    if db_model is None:
        return SchemasError(message="Data Not Found")
    bool_model = CRUD.update(db=db, pk=pk, item=item)
    return Schemas(data=SchemasResponse(**bool_model.to_dict()))


@router.patch("/{}/{{pk}}".format(name), name="update {}".format(name))
async def update_patch_model(pk: int, item: SchemasStoreUpdate, db: Session = Depends(dbs),
                             auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.update" % name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUD.first(db=db, pk=pk)
    if db_model is None:
        return SchemasError(message="Data Not Found")
    bool_model = CRUD.update(db=db, pk=pk, item=item, exclude_unset=True)
    return Schemas(data=SchemasResponse(**bool_model.to_dict()))


@router.delete("/{}/{{pk}}".format(name), name="delete {}".format(name))
async def delete_model(pk: int, db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.delete" % name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUD.delete(db=db, pk=pk)
    return Schemas(data=bool_model)


@router.delete("/{}".format(name), name="deletes {}".format(name))
async def delete_models(pks: List[int], db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.delete" % name])):
    """
    :param pks:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUD.delete(db=db, pks=pks)
    return Schemas(data=bool_model)



import json
from typing import Optional

from fastapi import Depends, Security
from fastapi.security import SecurityScopes, OAuth2PasswordBearer
from sqlalchemy.orm import Session

from config import OAUTH_TOKEN_URL, OAUTH_TOKEN_SCOPES, OAUTH_SECRET_KEY, OAUTH_ALGORITHM, OAUTH_LOGIN_SCOPES
from lsshu.internal.db import dbs
from lsshu.internal.helpers import token_payload
from lsshu.internal.schema import ModelScreenParams, ModelScreenParamsForAll
from lsshu.oauth.user.crud import CRUDOAuthUser
from lsshu.oauth.user.schema import SchemasOAuthUser, SchemasOAuthScopes


def model_screen_params(page: Optional[int] = 1, limit: Optional[int] = 25, quest_data: Optional[str] = None):
    """列表筛选参数"""
    order, where = [], []
    if bool(quest_data):
        quest_data = json.loads(quest_data) if quest_data else None
        [order.extend(list(s.items())) for s in quest_data['sort']] if 'sort' in quest_data else None
        where = [(w['key'], w['condition'], w['value']) for w in quest_data['where']] if 'where' in quest_data else None
    return ModelScreenParams(page=page, limit=limit, order=order, where=where)


def model_post_screen_params(data: ModelScreenParams = None):
    """列表筛选参数"""
    order = []
    [order.extend(list(s.items())) for s in data.order]
    data.order = order
    where = [(w['key'], w['condition'], w['value']) for w in data.where if
             ('value' in w and (w['value'] or w['value'] is False or w['value'] == 0)) and (not "join" in w or "join" in w and not w['join'])]
    join = [(w['join'], [(w['key'], w['condition'], w['value'])], 'join') for w in data.where if
            ('value' in w and (w['value'] or w['value'] is False or w['value'] == 0)) and ("join" in w and w['join'])]
    data.where = where
    data.join = join
    return data


def model_post_screen_params_for_all(data: ModelScreenParamsForAll = None):
    """列表筛选参数"""
    order = []
    [order.extend(list(s.items())) for s in data.order]
    data.order = order
    where = [(w['key'], w['condition'], w['value']) for w in data.where if
             ('value' in w and (w['value'] or w['value'] is False or w['value'] == 0)) and (not "join" in w or "join" in w and not w['join'])]
    join = [(w['join'], [(w['key'], w['condition'], w['value'])], 'join') for w in data.where if
            ('value' in w and (w['value'] or w['value'] is False or w['value'] == 0)) and ("join" in w and w['join'])]
    data.where = where
    data.join = join
    return data


oauth2_scheme = OAuth2PasswordBearer(tokenUrl=OAUTH_TOKEN_URL, scopes=OAUTH_TOKEN_SCOPES)


async def current_user_security(security_scopes: SecurityScopes, token: str = Depends(oauth2_scheme)):
    """
    解析加密字段
    :param security_scopes:
    :param token:
    :return:
    """
    payload = token_payload(security_scopes, token, OAUTH_SECRET_KEY, OAUTH_ALGORITHM)
    """处理授权用户实时情况"""
    # Todo
    """处理授权用户实时情况"""
    return SchemasOAuthUser(**payload)


async def auth_user(auth: SchemasOAuthUser = Security(current_user_security, scopes=[OAUTH_LOGIN_SCOPES]),
                    db: Session = Depends(dbs)):
    """
    demo
    :param auth:
    :param db:
    :return:
    """
    user = CRUDOAuthUser.first(db=db, pk=auth.user_id)
    return SchemasOAuthScopes(user=user, scopes=auth.scopes)



import os

ROOT_PATH = os.path.dirname(__file__)
# API 接口返回数据
SCHEMAS_SUCCESS_CODE: int = 0
SCHEMAS_SUCCESS_STATUS: str = 'success'
SCHEMAS_SUCCESS_MESSAGE: str = '数据请求成功！'
SCHEMAS_ERROR_CODE: int = 1
SCHEMAS_ERROR_STATUS: str = 'error'
SCHEMAS_ERROR_MESSAGE: str = '数据请求失败！'

# 站点
HOST_URL: str = ""

# 上传目录
UPLOAD_NAME: str = "static"
UPLOAD_DIR: str = "static"
UPLOAD_URI: str = "/static"

# OAuth 授权相关
OAUTH_DEFAULT_TAGS: list = ['OAuth']
OAUTH_LOGIN_SCOPES: str = "login"

OAUTH_TOKEN_URI: str = "/token"
OAUTH_TOKEN_URL: str = "/api%s" % OAUTH_TOKEN_URI
OAUTH_SCOPES_URI: str = "/scopes"
OAUTH_ME_URI: str = "/me"
OAUTH_TOKEN_SCOPES: dict = {
    OAUTH_LOGIN_SCOPES: OAUTH_LOGIN_SCOPES.capitalize()
}
OAUTH_SECRET_KEY: str = "4a876f7766d1a0e9d97231089be927e38d6dea09233ad212f056b7f1a75cd41d"
OAUTH_ALGORITHM: str = "HS256"
OAUTH_ACCESS_TOKEN_EXPIRE_MINUTES: int = 300
OAUTH_OAUTH_ROUTER: dict = {}

# 超级管理员 账号:密码
OAUTH_ADMIN_USERS: dict = {
    "admin": "admin"
}

# 数据库相关
DB_SQLALCHEMY_DATABASE_URL: str = "sqlite:///{}".format(os.path.join(ROOT_PATH, 'db.sqlite3'))
# DB_SQLALCHEMY_DATABASE_URL = "mysql+pymysql://root:root@192.168.1.3:3306/ic" # ic 数据库

# echo=True表示引擎将用repr()函数记录所有语句及其参数列表到日志
# 由于SQLAlchemy是多线程，指定check_same_thread=False来让建立的对象任意线程都可使用。这个参数只在用SQLite数据库时设置
DB_ENGINE_KWARGS: dict = {
    # "echo": True,
    # "encoding": 'utf-8',
    # "pool_pre_ping": True,
    # "pool_size": 100,
    # "pool_recycle": 3600,
    # "max_overflow": 100,
    "connect_args": {
        'check_same_thread': False,
        # "charset": "utf8mb4"
    }
}
# 在SQLAlchemy中，CRUD都是通过会话(session)进行的，所以我们必须要先创建会话，每一个SessionLocal实例就是一个数据库session
# flush()是指发送数据库语句到数据库，但数据库不一定执行写入磁盘；commit()是指提交事务，将变更保存到数据库文件
DB_SESSION_MAKER_KWARGS: dict = {
    "autoflush": False,
    "autocommit": False,
    "expire_on_commit": True
}



from datetime import datetime
from typing import Optional, List

from pydantic import BaseModel

from lsshu.internal.schema import SchemasPaginate


class SchemasResponse(BaseModel):
    """模型 返回"""
    id: int
    name: Optional[str] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None

    class Config:
        orm_mode = True


class SchemasStoreUpdate(BaseModel):
    """模型 提交"""
    name: Optional[str] = None


class SchemasPaginateItem(SchemasPaginate):
    items: List[SchemasResponse]


class SchemasParams(BaseModel):
    pass



from datetime import datetime
from typing import Optional, List

from pydantic import BaseModel

from lsshu.internal.schema import SchemasPaginate


class SchemasResponse(BaseModel):
    """模型 返回"""
    md5: Optional[str] = None
    path: Optional[str] = None
    preview_path: Optional[str] = None
    size: Optional[int] = None

    class Config:
        orm_mode = True


class SchemasStoreUpdate(BaseModel):
    """模型 提交"""
    filename: Optional[str] = None
    content_type: Optional[str] = None
    md5: Optional[str] = None
    path: Optional[str] = None
    size: Optional[int] = None
    width: Optional[int] = None
    height: Optional[int] = None



class SchemasPaginateItem(SchemasPaginate):
    items: List[SchemasResponse]


class SchemasParams(BaseModel):
    pass



from typing import List

from fastapi import APIRouter, Depends, Security, UploadFile, File
from sqlalchemy.orm import Session

from lsshu.internal.db import dbs
from lsshu.internal.depends import model_screen_params, model_post_screen_params, auth_user
from lsshu.internal.schema import ModelScreenParams, Schemas, SchemasError
from lsshu.oauth.user.schema import SchemasOAuthScopes

from lsshu.oauth.annex.crud import CRUD
from lsshu.oauth.model import annex_name as name
from lsshu.oauth.annex.schema import SchemasResponse, SchemasParams, SchemasPaginateItem, SchemasStoreUpdate

router = APIRouter(tags=["Annex"])
scopes = [name, ]


@router.get('/{}'.format(name), name="get {}".format(name))
async def get_models(db: Session = Depends(dbs), params: ModelScreenParams = Depends(model_screen_params),
                     auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.list" % name])):
    """
    :param db:
    :param params:
    :param auth:
    :return:
    """
    db_model_list = CRUD.paginate(db=db, screen_params=params)
    return Schemas(data=SchemasPaginateItem(**db_model_list))


@router.post('/{}.post'.format(name), name="get {}".format(name))
async def get_models(db: Session = Depends(dbs), params: ModelScreenParams = Depends(model_post_screen_params),
                     auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.list" % name])):
    """
    :param db:
    :param params:
    :param auth:
    :return:
    """
    db_model_list = CRUD.paginate(db=db, screen_params=params)
    return Schemas(data=SchemasPaginateItem(**db_model_list))


@router.get('/{}.params'.format(name), name="get {}".format(name))
async def params_models(db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.list" % name])):
    """
    :param db:
    :param auth:
    :return:
    """

    data = {}
    return Schemas(data=SchemasParams(**data))


@router.get('/{}/{{pk}}'.format(name), name="get {}".format(name))
async def get_model(pk: int, db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.get" % name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUD.first(db=db, pk=pk)
    if db_model is None:
        return SchemasError(message="Data Not Found")
    return Schemas(data=SchemasResponse(**db_model.to_dict()))


@router.post('/{}'.format(name), name="get {}".format(name))
async def store_model(file: UploadFile = File(...), db: Session = Depends(dbs),
                      auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.store" % name])):
    """
    :param file:
    :param db:
    :param auth:
    :return:
    """
    import os
    from config import UPLOAD_DIR
    from lsshu.internal.method import md5_bytes, write_file
    content = await file.read()
    md5 = md5_bytes(content)
    size = len(content)

    db_model = CRUD.first(db=db, where=("md5", md5))
    if not db_model:
        path = os.path.join(UPLOAD_DIR, os.path.split(file.content_type)[0], "{}{}".format(md5, os.path.splitext(file.filename)[-1]))
        bool_res = write_file(path, content)
        if bool_res:
            try:
                from PIL import Image
                img = Image.open(path)
                width, height = img.size
            except:
                width, height = (0, 0)
            db_model = CRUD.store(db=db,
                                  item=SchemasStoreUpdate(filename=file.filename, content_type=file.content_type, md5=md5, path=path, size=size, width=width, height=height))
    return Schemas(data=SchemasResponse(**db_model.to_dict()))


@router.put("/{}/{{pk}}".format(name), name="update {}".format(name))
async def update_put_model(pk: int, item: SchemasStoreUpdate, db: Session = Depends(dbs),
                           auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.update" % name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUD.first(db=db, pk=pk)
    if db_model is None:
        return SchemasError(message="Data Not Found")
    bool_model = CRUD.update(db=db, pk=pk, item=item)
    return Schemas(data=SchemasResponse(**bool_model.to_dict()))


@router.patch("/{}/{{pk}}".format(name), name="update {}".format(name))
async def update_patch_model(pk: int, item: SchemasStoreUpdate, db: Session = Depends(dbs),
                             auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.update" % name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUD.first(db=db, pk=pk)
    if db_model is None:
        return SchemasError(message="Data Not Found")
    bool_model = CRUD.update(db=db, pk=pk, item=item, exclude_unset=True)
    return Schemas(data=SchemasResponse(**bool_model.to_dict()))


@router.delete("/{}/{{pk}}".format(name), name="delete {}".format(name))
async def delete_model(pk: int, db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.delete" % name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUD.delete(db=db, pk=pk)
    return Schemas(data=bool_model)


@router.delete("/{}".format(name), name="deletes {}".format(name))
async def delete_models(pks: List[int], db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=scopes + ["%s.delete" % name])):
    """
    :param pks:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUD.delete(db=db, pks=pks)
    return Schemas(data=bool_model)



from sqlalchemy import Column, String, Boolean, Text, Integer
from sqlalchemy_mptt.mixins import BaseNestedSets

from lsshu.internal.db import Model, Engine


class _ModelOAuthUsers(Model, BaseNestedSets):
    """登录用户"""
    __abstract__ = True
    username = Column(String(15), nullable=False, unique=True, index=True, comment="名称")
    password = Column(String(128), nullable=False, comment="密码")
    user_phone = Column(String(11), nullable=True, comment="手机")
    available = Column(Boolean, default=1, comment="是否有效")


class _ModelOAuthRoles(Model, BaseNestedSets):
    """角色"""
    __abstract__ = True
    name = Column(String(15), nullable=False, unique=True, comment="名称")
    scopes = Column(Text, nullable=False, comment="Scope")


class _ModelOAuthPermissions(Model, BaseNestedSets):
    """ 权限 """
    __abstract__ = True
    name = Column(String(15), nullable=False, comment="名称")
    icon = Column(String(20), nullable=True, comment="ICO")
    scope = Column(String(50), nullable=False, unique=True, comment="Scope")
    path = Column(String(50), nullable=False, comment="Path")
    is_menu = Column(Boolean, default=True, comment="是否为菜单")
    is_action = Column(Boolean, default=True, comment="是否为动作权限")


class _ModelOAuthAnnexes(Model):
    """ 附件 """
    __abstract__ = True
    filename = Column(String(50), nullable=False, comment="文件名")
    content_type = Column(String(100), nullable=False, comment="类型")
    path = Column(String(100), nullable=True, comment="路径")
    md5 = Column(String(32), nullable=True, comment="md5", index=True)
    size = Column(Integer, nullable=False, comment="SIZE")
    width = Column(String(100), nullable=True, comment="宽")
    height = Column(String(190), nullable=True, comment="高")



import base64
import json

from pydantic import BaseModel
from typing import Optional, Union
from fastapi import WebSocket, WebSocketDisconnect

from lsshu.internal.redis import Redis, PickleSetObject


class WsState:
    """定义返回"""
    not_event = -1
    not_event_status = "error"
    success_code = 0
    success_status = "success"


class WsResponse(BaseModel):
    """状态返回"""
    code: Optional[int] = WsState.success_code
    status: Optional[str] = WsState.success_status
    event: Optional[str] = None
    data: Optional[Union[dict, list, str, None]] = None


class PickleSocket(PickleSetObject):
    socket = None


class WsEvents(object):
    self_socket: WebSocket = None
    redis: Redis = None

    def __init__(self, socket: WebSocket = None, redis: Redis = None):
        self.self_socket = socket
        self.redis = redis if redis else Redis()

    async def init(self):
        """
        分配事件给处理方法
        :param data:
        :return:
        """
        await self.self_socket.accept()
        try:
            while True:
                data = await self.self_socket.receive_text()
                data = self.decode(data)
                event = data.get('event', 'None')
                if hasattr(self, event):
                    data = data.get('data', None)
                    handle_data = await getattr(self, event)(data)
                    handle_data['event'] = handle_data.get("event", event)
                    await self.res_message(**handle_data)
                else:
                    await self.res_message(**data, code=WsState.not_event, status=WsState.not_event_status)
        except WebSocketDisconnect:
            pass

    def encode(self, data: dict) -> str:
        """
        字典转字符串 转base64
        :param data:
        :return:
        """
        return "lsshuey%s" % base64.b64encode(json.dumps(data).encode('utf-8')).decode('utf-8')

    def decode(self, data: str) -> dict:
        """
        base64转json json转字典
        :param data:
        :return:
        """
        return json.loads(base64.b64decode(data[7:]).decode('utf-8'))

    def response(self, data=None, **kwargs) -> dict:
        """
        返回数据
        :param data:
        :param kwargs:
        :return:
        """
        return WsResponse(**kwargs, data=data).dict()

    async def auth(self, data) -> dict:
        """连接"""
        ps = PickleSocket(data)
        await self.redis.pickle_set("all-socket", ps)
        return {"data": "ok"}

    async def res_message(self, socket: WebSocket = None, **kwargs):
        """连接"""
        response_data = self.response(**kwargs)
        response = self.encode(response_data)
        await (self.self_socket if not socket else socket).send_text(response)

    async def message(self, data):
        """连接"""
        # await self.manager.broadcast(data)
        return {"data": ""}



import aioredis
import pickle
from typing import Optional, Union, List, Tuple
from fastapi import FastAPI

try:
    from config import REDIS_CONFIG
except:
    REDIS_CONFIG = {
        "url": "redis://127.0.0.1:6379",
        "db": 10,
        "encoding": "utf-8",
        # "decode_responses": True
    }


class PickleSetObject(object):
    name = None

    def __init__(self, name: str):
        self.name = name


class Redis(object):
    key_prefix = None
    pool = None
    redis = None

    def __init__(self):
        self.pool = aioredis.ConnectionPool.from_url(**REDIS_CONFIG)
        self.redis = aioredis.Redis(connection_pool=self.pool)
        # """
        # https://blog.csdn.net/RoninYang/article/details/121128050
        # https://www.jianshu.com/p/e7f86bc09674?utm_campaign=maleskine
        # """
        # self.redis = app.state.redis = aioredis.from_url(**REDIS_CONFIG)
        #
        # @app.on_event("shutdown")
        # async def shutdown_event():
        #     app.state.redis.close() or app.state.redis.wait_closed()

    def __key(self, key, _type):
        """
        前缀键
        :param key:
        :param _type:
        :return:
        """
        return "%s-%s-%s" % (self.key_prefix, _type, key)

    def __value(self, value: Union[bytes, Tuple[bytes], List[bytes]], decode=True):
        """
        统一返回
        :param value:
        :param decode:
        :return:
        """
        return value if not decode else (
            value.decode() if type(value) is bytes else ([val.decode() for val in value] if type(value) is list else tuple([val.decode() for val in value])))

    async def string(self, key: str, value=None, default=None, decode=True):
        """
        设置字符串 或者 获取字符串
        :param key:
        :param value:
        :param default:
        :param decode:
        :return:
        """
        return await self.redis.set(self.__key(key, "string"), value) if value else (self.__value(await self.redis.get(self.__key(key, "string")), decode=decode) or default)

    async def hash(self, key: str, value: dict = None, field: Optional[Union[str, list]] = None, default=None, decode=True):
        """
        设置字符串 或者 获取 hash
        :param key:
        :param value:
        :param field:
        :param default:
        :param decode:
        :return:
        """
        if value:
            return await self.redis.hmset(self.__key(key, "hash"), value)
        else:
            data = self.__value(await self.redis.hmget(self.__key(key, "hash"), field), decode=decode)
            return (data if type(field) == list else data.pop()) or default

    async def list(self, key: str, value=None, default: list = [], start: int = 0, end: int = 10, decode=True):
        """
        设置或者获取 列表
        :param key:
        :param value:
        :param default:
        :param start:
        :param end:
        :param decode:
        :return:
        """
        return await self.redis.lpush(self.__key(key, "list"), value) if value else (
                self.__value((await self.redis.lrange(self.__key(key, "list"), start, end)), decode=decode) or default)

    async def list_remove(self, key: str, count, value):
        """
        移除列表的值
        :param key:
        :param count:
        :param value:
        :return:
        """
        return await self.redis.lrem(self.__key(key, "list"), count=count, value=value)

    async def set(self, key: str, value=None, default: tuple = tuple(), decode=True):
        """
        设置或者获取 列表
        :param key:
        :param value:
        :param default:
        :param decode:
        :return:
        """
        return await self.redis.sadd(self.__key(key, "set"), value) if value else (self.__value((await self.redis.smembers(self.__key(key, "set"))), decode=decode) or default)

    async def set_remove(self, key: str, value):
        """
        移除SET的值
        :param key:
        :param value:
        :return:
        """
        return await self.redis.smove(self.__key(key, "set"), value=value)

    async def sets(self, key: str, *value):
        """
        设置列表
        :param key:
        :param value:
        :return:
        """
        return await self.redis.sadd(self.__key(key, "set"), *value)

    async def pickle(self, key: str, value: object = None, default=None):
        """
        设置字符串 或者 获取字符串
        :param key:
        :param value:
        :param default:
        :return:
        """
        return await self.redis.set(self.__key(key, "pickle"), pickle.dumps(value)) if value else (pickle.loads(await self.redis.get(self.__key(key, "pickle"))) or default)

    async def pickle_set(self, key: str, value: PickleSetObject = None):
        """
        设置 pickle set
        :param key:
        :param value:
        :return:
        """
        if value:
            await self.set(self.__key(key, "pickle_set"), value.name)
            return await self.pickle(self.__key(value.name, "pickle_set"), value=value)
        else:
            keys = await self.set(self.__key(key, "pickle_set"))
            return tuple([await self.pickle(self.__key(se, "pickle_set")) for se in keys])

    async def delete(self, key: str, _type: str = "string"):
        """
        删除键值
        :param key:
        :param _type:
        :return:
        """
        return await self.redis.delete(self.__key(key, _type))

    async def disconnect(self):
        """
        断开链接
        :return:
        """
        await self.pool.disconnect()


async def _redis():
    redis = Redis()
    try:
        yield redis
    finally:
        await redis.disconnect()



"""
@Time    ：2022/10/31 10:04
@Author  ：Lsshu
@File    ：setup.py
@Version ：2.0.9
@Project ：lsshu-cms
"""
from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(name="lsshu-cms",
      version="2.1.0",
      description="FastAPI 开发的CMS",
      python_requires=">=3.9",
      author="Lsshu",
      author_email="admin@lsshu.cn",
      url="https://github.com/lsshu/fastapi-cms",
      packages=find_packages(),
      long_description=long_description,
      long_description_content_type="text/markdown",
      license="GPLv3",
      classifiers=[
          "Programming Language :: Python :: 3.9",
          "License :: OSI Approved :: MIT License",
          "Development Status :: 3 - Alpha",
      ],
      install_requires=[
          "hashids",
          "fastapi[all]",
          "python-jose[cryptography]",
          "passlib[bcrypt]",
          "SQLAlchemy==1.4.46",
          "Pillow",
          "pymysql",
          "sqlalchemy-mptt",
          "user-agents",
          "requests",
          "openpyxl",
          "aioredis",
          "filetype",
          "pycryptodome"
      ]
      )



from sqlalchemy import Column, Integer, ForeignKey, Table, event
from sqlalchemy.orm import relationship

from lsshu.internal.method import plural
from lsshu.internal.model import Model, Engine, _ModelOAuthUsers, _ModelOAuthRoles, _ModelOAuthPermissions, _ModelOAuthAnnexes

_name = "oauth".capitalize()
_table_name = _name.replace('.', '_')
user_name = plural("%s.user" % _name)
role_name = plural("%s.role" % _name)
permission_name = plural("%s.permission" % _name)
annex_name = plural("%s.annex" % _name)

user_table_name = user_name.replace('.', '_')
permission_table_name = permission_name.replace('.', '_')
role_table_name = role_name.replace('.', '_')
annex_table_name = annex_name.replace('.', '_')

SYSTEM_PERMISSIONS = [
    {
        "name": "后台管理",
        "scope": _name,
        "icon": "ElementPlus",
        "children": [
            {"name": "授权用户", "scope": user_name, "icon": "User"},
            {"name": "用户角色", "scope": role_name, "icon": "Money"},
            {"name": "权限管理", "scope": permission_name, "icon": "KnifeFork", "action": [{"name": "菜单", "scope": "menus"}, {"name": "树", "scope": "tree"}]},
            {"name": "附件文件", "scope": annex_name, "icon": "Folder"},
        ]
    }
]


class ModelOAuthUsers(_ModelOAuthUsers):
    """登录用户"""
    __tablename__ = user_table_name
    permissions = relationship('ModelOAuthPermissions', backref='auth_users', secondary=Table(
        "%s_user_has_permissions" % _table_name,
        Model.metadata,
        Column('per_id', Integer, ForeignKey("%s.id" % permission_table_name), primary_key=True, comment="权限"),
        Column('use_id', Integer, ForeignKey("%s.id" % user_table_name), primary_key=True, comment="用户"),
    ))
    roles = relationship('ModelOAuthRoles', backref='auth_users', secondary=Table(
        "%s_user_has_roles" % _table_name,
        Model.metadata,
        Column('rol_id', Integer, ForeignKey("%s.id" % role_table_name), primary_key=True, comment="角色"),
        Column('use_id', Integer, ForeignKey("%s.id" % user_table_name), primary_key=True, comment="用户"),
    ))
    sort = Column(Integer, default=999, comment="排序")


class ModelOAuthRoles(_ModelOAuthRoles):
    """角色"""
    __tablename__ = role_table_name
    permissions = relationship('ModelOAuthPermissions', backref='roles', lazy="joined", secondary=Table(
        "%s_role_has_permissions" % _table_name,
        Model.metadata,
        Column('per_id', Integer, ForeignKey("%s.id" % permission_table_name), primary_key=True, comment="权限"),
        Column('rol_id', Integer, ForeignKey("%s.id" % role_table_name), primary_key=True, comment="角色")
    ))


class ModelOAuthPermissions(_ModelOAuthPermissions):
    """ 权限 """
    __tablename__ = permission_table_name


@event.listens_for(ModelOAuthPermissions.scope, 'set')
def receive_before_insert(target, value: str, old_value, initiator):
    target.path = "/%s" % value.replace('.', '/').lower()


@event.listens_for(ModelOAuthPermissions.scope, 'modified')
def receive_before_insert(target, initiator):
    target.path = "/%s" % target.scope.replace('.', '/').lower()


class ModelOAuthAnnexes(_ModelOAuthAnnexes):
    """ 附件 """
    __tablename__ = annex_table_name

    @property
    def preview_path(self):
        import os
        from config import HOST_URL
        return os.path.join(HOST_URL, self.path)



from datetime import datetime
from typing import Optional, Union, List

from pydantic import BaseModel

from lsshu.internal.schema import SchemasPaginate, Schemas


class SchemasOAuthPermissionResponse(BaseModel):
    """权限 返回"""
    id: int
    name: Optional[str] = None
    icon: Optional[str] = None
    scope: Optional[str] = None
    path: Optional[str] = None
    parent_id: Optional[int] = None
    is_menu: Optional[bool] = True
    is_action: Optional[bool] = True
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None

    class Config:
        orm_mode = True


class SchemasOAuthPermissionThinResponse(BaseModel):
    """权限 返回"""
    id: int
    name: Optional[str] = None

    class Config:
        orm_mode = True


class SchemasOAuthPermissionPaginateItem(SchemasPaginate):
    items: List[SchemasOAuthPermissionResponse]


class SchemasOAuthPermissionStoreUpdate(BaseModel):
    """授权角色 提交"""
    name: Optional[str] = None
    icon: Optional[str] = None
    scope: Optional[str] = None
    path: Optional[str] = None
    parent_id: Optional[int] = None
    is_menu: Optional[bool] = True
    is_action: Optional[bool] = True


class SchemasOAuthPermissionTreeResponse(SchemasOAuthPermissionResponse):
    """权限 返回"""
    children: Optional[list] = None


class SchemasOAuthPermissionTreeStatusResponse(Schemas):
    """状态树返回"""
    data: List[SchemasOAuthPermissionTreeResponse]


class SchemasOAuthPermissionMenu(BaseModel):
    """权限 返回"""
    id: int
    name: Optional[str] = None
    icon: Optional[str] = None
    scope: Optional[str] = None
    path: Optional[str] = None

    class Config:
        orm_mode = True


class SchemasOAuthPermissionMenuResponse(SchemasOAuthPermissionMenu):
    """菜单 返回"""
    children: Optional[List[SchemasOAuthPermissionMenu]] = None


class SchemasOAuthPermissionMenuStatusResponse(Schemas):
    """菜单树返回"""
    data: List[SchemasOAuthPermissionMenuResponse]
